@@ -1,10 +1,26 @@
+""" ItemStore.py
+
+Data store abstraction module.
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """
+Straw is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+Straw is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
 import cPickle as pickle
 import os, sys
 import time
 from error import *
-import pygtk
-pygtk.require('2.0')
-import gtk
 import tempfile
 import traceback
 
@@ -15,11 +31,11 @@ except ImportError:
     from bsddb3.db import *
     import bsddb3 as bsddb
 
-import FeedList
 import Event
 import SummaryItem
-import hig_alert
+import FeedList
 import ImageCache
+from MainloopManager import MainloopManager
 
 DATABASE_FILE_NAME = "itemstore.db"
 
@@ -30,37 +46,38 @@ class ConvertException(Exception):
         self.reason = reason
 
 class MyDB:
-    CURRENT_VERSION = 2
-    
+    CURRENT_VERSION = 3
+
     def __init__(self, filename, dbhome, create=0, truncate=0, mode=0600,
                  recover=0, dbflags=0):
         self._db = None
         self._env = None
         recoverenv = DB_CREATE | DB_RECOVER
-        flagsforenv = DB_INIT_MPOOL | DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_TXN
+        # DB_INIT_TXN automatically enables logging
+        flagsforenv = DB_INIT_TXN | DB_INIT_MPOOL | DB_INIT_LOCK | DB_PRIVATE
 
         self._env = DBEnv()
+        self._env.set_data_dir(dbhome)
         self._env.set_lk_detect(DB_LOCK_DEFAULT)  # enable auto deadlock avoidance
         self._env.set_lg_max(2**20)
-        #self._env.set_cachesize(0, 600000, 1)
 
         try:
-            self._env.open(dbhome, flagsforenv | recoverenv, mode)
+            self._env.open(dbhome, recoverenv | flagsforenv, mode)
         except bsddb._db.DBRunRecoveryError, err:
             self._env.remove(dbhome)
             self._env.close()
             log("%s" % err[1])
             sys.exit("Recovery Error: See README for details on how to recover data. ")
 
+        flags = 0
+        if truncate:
+            flags |= DB_TRUNCATE
+
         try:
-            dbflags |= DB_AUTO_COMMIT
+            flags |= DB_AUTO_COMMIT
         except NameError:
             pass
 
-        myflags = 0
-        if truncate:
-            myflags = myflags | DB_TRUNCATE
-        flags = dbflags | myflags
         try:
             self._db = DB(self._env)
             self._db.open(filename, DB_BTREE, flags, mode)
@@ -74,20 +91,14 @@ class MyDB:
         try:
             self.convert_old_versions()
         except Exception, ex:
-            filename = tempfile.mktemp(prefix="straw-")
-            fh = open(filename, "w")
-            traceback.print_exc(None, fh)
-            fh.close()
-            hig_alert.reportError(
-                _("There was a problem while converting the database"),
-                _("""
-                Straw will not behave as excepted, you should probably quit now.
-                The exception has been saved to the file %s. Please see
-                the Straw README for further instructions.
-                """) % filename)
-
-    def __del__(self):
-        self.close()
+            try:
+                filename = tempfile.mktemp(prefix="straw-")
+                fh = open(filename, "w")
+                traceback.print_exc(None, fh)
+                raise ConvertException(self.get_db_version(),
+                                       MyDB.CURRENT_VERSION, "%s" % filename)
+            finally:
+                fh.close()
 
     def close(self):
         if self._db is not None:
@@ -108,18 +119,32 @@ class MyDB:
     def begin_transaction(self):
         return self._env.txn_begin()
 
-    def get_feed_item_ids(self, feed, txn=None):
+    def get_feed_item_ids(self, feed, txn):
         key = "fids:%d" % feed.id
-        return self.__get_item_ids(key, txn)
+        ids = self._get_item_ids(key, txn)
+        return ids
+        #log("TEMPORARY HACK: filtering out duplicate ids from feed item id list. This should be disabled (and unnecessary) before release.")
+        #seen = {}
+        #realids = []
+        #for itemid in ids:
+        #    seen[itemid] = seen.get(itemid, 0) + 1
+        #    if seen[itemid] > 1:
+        #        log("Found duplicate id ", itemid, ", filtering")
+        #        continue
+        #    realids.append(itemid)
+        #if len(realids) < len(ids):
+        #    log("Saving the new feed id list")
+        #    self.save_feed_item_ids(feed, realids, txn)
+        #return realids
 
-    def get_item_ids_for_id(self, iid, txn=None):
+    def get_item_ids_for_id(self, iid, txn):
         # Used for low-level stuff
         # Hopefully, this will be used in the future for more separation
         # Currently used by config conversion
         key = "fids:%d" % iid
-        return self.__get_item_ids(key, txn)
+        return self._get_item_ids(key, txn)
 
-    def __get_item_ids(self, key, txn):
+    def _get_item_ids(self, key, txn):
         dids = self._db.get(key, txn=txn)
         ids = []
         if dids:
@@ -129,7 +154,7 @@ class MyDB:
     def save_feed_item_ids(self, feed, ids, txn=None):
         rowid = "fids:%d" % feed.id
         commit = 0
-        if txn is None:
+        if not txn:
             txn = self.begin_transaction()
             commit = 1
         try:
@@ -138,30 +163,23 @@ class MyDB:
             except DBNotFoundError:
                 pass
             self._db.put(rowid, pickle.dumps(ids), txn=txn)
-            if commit:
-                txn.commit()
-        except:
+        except Exception, ex:
             if commit:
                 txn.abort()
-            raise
+            logtb(str(ex))
+        else:
+            if commit:
+                txn.commit()
 
     def get_item(self, feed, item_id, txn=None):
-        return self.__get_item(feed.id, item_id, txn)
+        return self._get_item(feed.id, item_id, txn)
 
     def get_item_for_id(self, iid, item_id, txn=None):
-        # Used for low-level stuff
-        # Hopefully, this will be used in the future so we can separate
-        # mechanism from policy
-        # Currently by config conversion
-        return self.__get_item(iid, item_id, txn)
-
-    def __get_item(self, iid, item_id, txn=None):
-        item = unstringify_item(
-            self._db.get("%d:%d" % (iid, item_id), txn=txn))
-        if item is None:
-            return None
-        return item
+        return self._get_item(iid, item_id, txn)
 
+    def _get_item(self, iid, item_id, txn=None):
+        item = self._db.get("%d:%d" % (iid, item_id), txn=txn)
+        return unstringify_item(item)
 
     def add_item(self, item):
         txn = self.begin_transaction()
@@ -182,9 +200,12 @@ class MyDB:
         try:
             feed_item_ids = self.get_feed_item_ids(feed, txn=txn)
             for item in items:
+                self._db.put("%d:%d" % (item.feed.id, item.id), stringify_item(item), txn=txn)
+                # TODO: it might be a good idea to check here that we don't add
+                # duplicate items. It doesn't happen normally, but there can be
+                # bugs that trigger that. Throwing an exception would be the
+                # the right thing: it wouldn't hide the breakage.
                 feed_item_ids.append(item.id)
-                self._db.put("%d:%d" % (item.feed.id, item.id),
-                             stringify_item(item), txn=txn)
             self.save_feed_item_ids(feed, feed_item_ids, txn)
         except Exception, ex:
             txn.abort()
@@ -192,17 +213,38 @@ class MyDB:
         else:
             txn.commit()
 
-    def del_item(self, feed, item):
+    def del_item(self, feed, items):
+        log("DEPRECATED: use delete_items")
+        self.delete_items(feed, items)
+
+
+    def delete_items(self, feed, items):
+        """ Deletes a list of items.
+
+        Useful for cutting old items based on number of items stored.
+        """
         txn = self.begin_transaction()
         try:
             feed_item_ids = self.get_feed_item_ids(feed, txn=txn)
-            if item.id in feed_item_ids:
-                feed_item_ids.remove(item.id)
-                self.save_feed_item_ids(feed, feed_item_ids, txn)
-                self._db.delete("%d:%d" % (feed.id, item.id), txn=txn)
+            # because of bugs, we sometime get here duplicate ids. instead of dying,
+            # warn the user but continue
+            item_ids = []
+            for item in items:
+                item.clean_up()
+                if item.id in item_ids:
+                    log("WARNING: skipping duplicate ids in delete items request %s and %s" % (item.title, item.id))
+                    # filter out any duplicates
+                    feed_item_ids = filter(lambda x: x != item.id, feed_item_ids)
+                    continue
+                item_ids.append(item.id)
+                #log("deleting item %d:%d" % (feed.id, item.id))
+                if item.id in feed_item_ids:
+                    feed_item_ids.remove(item.id)
+                    self._db.delete("%d:%d" % (feed.id, item.id), txn=txn)
+            self.save_feed_item_ids(feed, feed_item_ids, txn)
         except Exception, ex:
             txn.abort()
-            logtb(str(ex))
+            log_exc("error while deleting items")
         else:
             txn.commit()
 
@@ -229,7 +271,7 @@ class MyDB:
                     items.append(item)
         except Exception, ex:
             txn.abort()
-            logtb(str(ex))
+            log(str(ex))
         else:
             txn.commit()
             return items
@@ -266,6 +308,33 @@ class MyDB:
     def save_image_urls(self, urls, txn=None):
         self._db.put("images", pickle.dumps(urls), txn=txn)
 
+    def get_image_counts(self, txn=None):
+        images = self.get_image_urls(txn)
+        counts = []
+        for image in images:
+            key = ("imagecount:" + image).encode('utf-8')
+            value = self._db.get(str(key))
+            try:
+                counts.append((image, int(value)))
+            except:
+                log("exception for ", key, ", type of value ", value, ": ", type(value))
+        return counts
+
+    def update_image_count(self, url, count):
+        #logparam(locals(), "url", "count")
+        key = ("imagecount:" + url).encode('utf-8')
+        txn = self.begin_transaction()
+        try:
+            if count < 1:
+                self._db.delete(key, txn=txn)
+            else:
+                self._db.put(key, str(count), txn=txn)
+        except:
+            txn.abort()
+            raise
+        else:
+            txn.commit()
+
     def update_image(self, url, image):
         key = "image:%s" % str(url)
         txn = self.begin_transaction()
@@ -278,13 +347,17 @@ class MyDB:
                     self.save_image_urls(image_urls, txn)
             else:
                 if url in image_urls:
-                    self._db.delete(key, txn=txn)
+                    try:
+                        self._db.delete(key, txn=txn)
+                    except DBNotFoundError:
+                        log("Key not found", key)
                     image_urls.remove(url)
                     self.save_image_urls(image_urls, txn=txn)
-            txn.commit()
         except:
             txn.abort()
             raise
+        else:
+            txn.commit()
 
     def get_image_data(self, url, txn=None):
         return self._db.get(
@@ -296,7 +369,7 @@ class MyDB:
 
     def _data_print(self, key, data):
         data = pickle.loads(data)
-        print `{key: data}`
+        pprint ({key: data})
 
     def _db_print(self, helper):
         """Print the database to stdout for debugging"""
@@ -310,10 +383,7 @@ class MyDB:
                 next = cur.next()
                 if next:
                     key, data = next
-                else:
-                    cur.close()
-                    return
-        except DBNotFoundError:
+        finally:
             cur.close()
 
     def get_db_version(self, txn=None):
@@ -321,14 +391,15 @@ class MyDB:
         return int(version)
 
     def set_db_version(self, version, txn=None):
-        if txn is None:
-            txn = self.begin_transaction()
         try:
-            self._db.put("straw_db_version", str(version), txn=txn)
-            txn.commit()
+            if txn is None:
+                txn = self.begin_transaction()
+                self._db.put("straw_db_version", str(version), txn=txn)
         except:
             txn.abort()
             raise
+        else:
+            txn.commit()
 
     def convert_old_versions(self):
         version = self.get_db_version()
@@ -360,8 +431,8 @@ class MyDB:
         except ImportError:
             raise ConvertException(1, 2, _("Couldn't import mx.DateTime"))
         txn = self.begin_transaction()
-        cur = self._db.cursor(txn)
         try:
+            cur = self._db.cursor(txn=txn)
             try:
                 next = cur.first()
                 key = None
@@ -389,6 +460,50 @@ class MyDB:
         else:
             txn.commit()
 
+    def convert_2_3(self):
+        def is_item(key):
+            parts = key.split(':')
+            if len(parts) != 2:
+                return False
+            return parts[0].isdigit() and parts[1].isdigit()
+
+        imagelistcursor = None
+        images = {}
+        txn = self.begin_transaction()
+        try:
+            cur = self._db.cursor(txn=txn)
+            try:
+                next = cur.first()
+                key = None
+                if next:
+                    key, data = cur.first()
+                while key is not None:
+                    if is_item(key):
+                        dic = pickle.loads(data)
+                        for image in dic['images']:
+                            images[image] = images.get(image, 0) + 1
+                    elif key == "images":
+                        imagelistcursor = cur.dup(DB_POSITION)
+                    next = cur.next()
+                    if next:
+                        key, data = next
+                    else:
+                        break
+                for image, count in images.items():
+                    key = ("imagecount:" + image).encode('utf-8')
+                    # TODO: remove debugging message
+                    log("storing count ", count, " for key ", key)
+                    cur.put(key, str(count), DB_KEYFIRST)
+                imagelistcursor.put("images", pickle.dumps(images.keys()), DB_CURRENT)
+            finally:
+                cur.close()
+                imagelistcursor.close()
+        except Exception, ex:
+            txn.abort()
+            raise
+        else:
+            txn.commit()
+
 class ModifyItemAction:
     def __init__(self, item):
         self._item = item
@@ -412,12 +527,12 @@ class ItemsAddedAction:
         db.add_items(self._feed, self._items)
 
 class DeleteItemAction:
-    def __init__(self, feed, item):
+    def __init__(self, feed, items):
         self._feed = feed
-        self._item = item
+        self._items = items
 
     def doit(self, db):
-        db.del_item(self._feed, self._item)
+        db.delete_items(self._feed, self._items)
 
 class ImageUpdateAction:
     def __init__(self, url, image):
@@ -427,32 +542,56 @@ class ImageUpdateAction:
     def doit(self, db):
         db.update_image(self._url, self._image)
 
+class ImageCountChangedAction:
+    def __init__(self, url, count):
+        self._url = url
+        self._count = count
+
+    def doit(self, db):
+        db.update_image_count(self._url, self._count)
+
 class ItemStore:
-    def __init__(self, straw_dir):
+    def __init__(self, dbhome):
         feedlist = FeedList.get_instance()
-        self._db = MyDB(DATABASE_FILE_NAME, straw_dir, create = 1)
+        self._db = MyDB(DATABASE_FILE_NAME, dbhome, create = 1)
         self.connect_signals()
-        feedlist.signal_connect(
-            Event.FeedsChangedSignal, self.feeds_changed)
+        feedlist.signal_connect(Event.FeedCreatedSignal,
+                                self._feed_created_cb)
+        feedlist.signal_connect(Event.FeedDeletedSignal,
+                                self._feed_deleted_cb)
         ImageCache.cache.signal_connect(Event.ImageUpdatedSignal,
-                                              self.image_updated)
-        self._stop = 0
+                                        self.image_updated)
+        self._stop = False
         self._action_queue = []
 
     #def initialize_db(self):
     #    self._db.CreateTable("items", ["feed_id", "item_id", "item_data"])
 
-    def feeds_changed(self, signal):
-        self.connect_signals()
+    def _feed_created_cb(self, signal):
+        self._connect_feed_signals(signal.feed)
+
+    def _feed_deleted_cb(self, signal):
+        self._disconnect_feed_signals(signal.feed)
 
     def connect_signals(self):
         flist = FeedList.get_instance().flatten_list()
         for f in flist:
-            f.signal_connect(Event.NewItemsSignal, self.items_added)
-            f.signal_connect(Event.ItemReadSignal, self.item_modified)
-            f.signal_connect(Event.ItemStickySignal, self.item_modified)
-            f.signal_connect(Event.AllItemsReadSignal, self.all_items_read)
-            f.signal_connect(Event.ItemDeletedSignal, self.item_deleted)
+            self._connect_feed_signals(f)
+
+    def _connect_feed_signals(self, feed):
+        feed.signal_connect(Event.NewItemsSignal, self.items_added)
+        feed.signal_connect(Event.ItemReadSignal, self.item_modified)
+        feed.signal_connect(Event.ItemStickySignal, self.item_modified)
+        feed.signal_connect(Event.AllItemsReadSignal, self.all_items_read)
+        feed.signal_connect(Event.ItemDeletedSignal, self.item_deleted)
+
+    def _disconnect_feed_signals(self, feed):
+        feed.signal_disconnect(Event.NewItemsSignal, self.items_added)
+        feed.signal_disconnect(Event.ItemReadSignal, self.item_modified)
+        feed.signal_disconnect(Event.ItemStickySignal, self.item_modified)
+        feed.signal_disconnect(Event.AllItemsReadSignal, self.all_items_read)
+        feed.signal_disconnect(Event.ItemDeletedSignal, self.item_deleted)
+
 
     def modify_item(self, item):
         self._action_queue.append(ModifyItemAction(item))
@@ -485,13 +624,25 @@ class ItemStore:
     def get_number_of_unread(self, feed_id, cutoff):
         return self._db.get_number_of_unread(feed_id, cutoff)
 
+    def get_image_counts(self):
+        return self._db.get_image_counts()
+
+    def set_image_count(self, image, count):
+        self._action_queue.append(
+            ImageCountChangedAction(image, count))
+
     def start(self):
-        gtk.timeout_add(5000, self.run)
+        mlmgr = MainloopManager.get_instance()
+        mlmgr.set_repeating_timer(5000, self._run)
 
     def stop(self):
-        self._stop = 1
+        mlmgr = MainloopManager.get_instance()
+        mlmgr.end_repeating_timer(self._run)
+        self._db.checkpoint()
+        self._db.close()
+        self._stop = True
 
-    def run(self):
+    def _run(self):
         self._db.checkpoint()
         freq = 5
         timer = freq
@@ -516,16 +667,13 @@ class ItemStore:
             if cptimer > cpfreq:
                 self._db.checkpoint()
                 cptimer = 0
-            gtk.timeout_add(freq * 1000, self.run)
-        else:
-            self._db.checkpoint()
-            self._db.close()
 
 itemstore_instance = None
-
 def get_instance(straw_dir=None):
     global itemstore_instance
     if itemstore_instance is None:
+        if straw_dir is None:
+            straw_dir = os.path.join(os.getenv('HOME'), ".straw")
         itemstore_instance = ItemStore(straw_dir)
     return itemstore_instance
 
@@ -555,38 +703,49 @@ def stringify_item(item):
     return pickle.dumps(itemdict)
 
 def unstringify_item(itemstring):
-    if itemstring is None:
+    if not itemstring:
         return None
-    try:
-        dict = pickle.loads(itemstring)
-    except ValueError, e:
-        log("ItemStore.unstringify_item: pickle.loads raised ValueError, argument was %s" % repr(itemstring))
+
+    idict = _unpickle(itemstring)
+    if not idict:
         return None
+
     item = SummaryItem.SummaryItem()
-    item.title = dict['title']
-    item.link = dict['link']
-    item.description = dict['description']
-    item.guid = dict['guid']
-    item.pub_date = dict['pub_date']
-    item.source = dict['source']
-    for i in dict['images']:
+    item.title = idict['title']
+    item.link = idict['link']
+    item.description = idict['description']
+    item.guid = idict['guid']
+    item.pub_date = idict['pub_date']
+    item.source = idict['source']
+    for i in idict['images']:
         item.restore_image(i)
-    item.seen = dict['seen']
-    item.id = dict['id']
-    item.guidislink = dict.get('guidislink', True)
-    item.fm_license = dict.get('fm_license', None)
-    item.fm_changes = dict.get('fm_changes', None)
-    item.creator = dict.get('creator', None)
-    item.contributors = dict.get('contributors', None)
-    item.license_urls = dict.get('license_urls', None)
-    item._sticky = dict.get('sticky', 0)
-    item.publication_name = dict.get('publication_name', None)
-    item.publication_volume = dict.get('publication_volume', None)
-    item.publication_number = dict.get('publication_number', None)
-    item.publication_section = dict.get('publication_section', None)
-    item.publication_starting_page = dict.get('publication_starting_page', None)
+    item.seen = idict['seen']
+    item.id = idict['id']
+    item.guidislink = idict.get('guidislink', True)
+    item.fm_license = idict.get('fm_license', None)
+    item.fm_changes = idict.get('fm_changes', None)
+    item.creator = idict.get('creator', None)
+    item.contributors = idict.get('contributors', None)
+    item.license_urls = idict.get('license_urls', None)
+    item._sticky = idict.get('sticky', 0)
+    item.publication_name = idict.get('publication_name', None)
+    item.publication_volume = idict.get('publication_volume', None)
+    item.publication_number = idict.get('publication_number', None)
+    item.publication_section = idict.get('publication_section', None)
+    item.publication_starting_page = idict.get('publication_starting_page', None)
     return item
 
+def _unpickle(istring):
+    itemdict = None
+    try:
+        itemdict = pickle.loads(istring)
+    except ValueError, ve:
+        log("ItemStore.unstringify_item: pickle.loads raised ValueError, argument was %s" % repr(itemstring))
+    except Exception, ex:
+        logtb(str(ex))
+    return itemdict
+
 if __name__ == '__main__':
+    from pprint import pprint
     db = MyDB("itemstore.db", "%s/.straw" % os.getenv('HOME'), create = 1)
     db._db_print(db._data_print)
