@@ -1,62 +1,37 @@
-import re, string, htmlentitydefs, urllib, urlparse, locale, time, calendar
-import os, sys
+""" utils.py
+
+Module for utility methods.
+"""
+__copyright__ = "Copyright (c) 2002-2005 Free Software Foundation, Inc."
+__license__ = """GNU General Public License
+
+This program is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place - Suite 330, Boston, MA 02111-1307, USA. """
+
+import re, string, htmlentitydefs
+from htmlentitydefs import codepoint2name
+import urllib, urlparse, locale
+import time, calendar
+import os
+import os.path
+import subprocess
+import sys
 import error
+import constants
+import gtk
 
-url_re = re.compile('^\w+://')
 entity = re.compile(r'\&.\w*?\;')
-html_tags = re.compile(r'\<.*?\>')
 
-def is_url(text):
-    return url_re.match(text)
-
-def read_text(fragment, chars):
-    """Read chars cdata characters from html fragment fragment"""
-    count = 0
-    in_tag = 0
-    in_attr = 0
-    in_entity = 0
-    res_len = 0
-    if fragment is None:
-        return ""
-    frag_len = len(fragment)
-    i = -1
-    res = []
-    previous = None
-    while res_len < chars and i < frag_len - 1:
-        i += 1
-        current = fragment[i]
-        if in_tag:
-            if in_attr:
-                if current == '"':
-                    in_attr = 0
-            else:
-                if current == '"':
-                    in_attr = 1
-                elif current == '>':
-                    in_tag = 0
-        elif in_entity:
-            res.append(current)
-            if current == ';':
-                in_entity = 0
-                previous = ""
-                res_len += 1
-        else:
-            if current == "<":
-                in_tag = 1
-            elif current == "&":
-                in_entity = 1
-                res.append("&")
-            # avoid beginning whitespace, newlines and returns, and multiple
-            # whitespaces
-            elif len(res) or current not in string.whitespace:
-                if current == '\n' or current == '\r':
-                    current = ' '
-                    if current == previous == ' ':
-                        continue
-                res.append(current)
-                previous = current
-                res_len += 1
-    return "".join(res)
 
 def convert_entities(text):
     def conv(ents):
@@ -92,11 +67,6 @@ def convert_entities(text):
         ctext = in_entity.re.sub(conv, text)
         return ctext
 
-def is_html(text):
-    is_tag = html_tags.search(text)
-    if is_tag: return 1
-    else: return 0
-
 def complete_url(url, feed_location):
     url = urllib.quote(url, safe=string.punctuation)
     if urlparse.urlparse(url)[0] == '':
@@ -121,6 +91,8 @@ def format_date(date, format=None, encoding=None):
         format = get_date_format()
     if encoding is None:
         encoding = get_locale_encoding()
+    if encoding.find('.') > 0:
+        encoding = encoding.split('.')[1] # get the encoding
     timestr = time.strftime(format, time.localtime(calendar.timegm(date)))
     return unicode(timestr, encoding)
 
@@ -139,56 +111,40 @@ def get_date_format():
     # it's necessary in the translation file.
     return _('%A %B %e %H:%M')
 
-MAX_DISPLAY_CHAR = 60
-def convert_title(title, description = ''):
-    if title is None or title == '':
-        try:
-            if len(description):
-                title = read_text(description, MAX_DISPLAY_CHAR) + '...'
-        except TypeError:
-            error.log("empty title and description, leave as is...")
-            # no title, no description, leave it.
-            return
-
-    title = convert_entities(title)
-
-    if is_html(title):
-        title = read_text(title, MAX_DISPLAY_CHAR)
-
-    return title
-
-def find_lib_dir():
+def find_data_dir():
+    datad = ''
     if os.environ.has_key("STRAW_DATA"):
-        return os.environ["STRAW_DATA"]
-    h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
-    if t == 'bin':
-        fp = os.path.join(h, 'share', 'straw')
-        if os.path.isdir(fp):
-            return fp
-        fp = os.path.join(h, 'lib', 'straw')
-        if os.path.isdir(fp):
-            return fp
-    raise "FileNotFoundError", "couldn't find Straw data directory"
+        datad = os.environ["STRAW_DATA"]
+    else:
+        datad = constants.datadir
+        if not os.path.isdir(datad):
+            raise "FileNotFoundError", "couldn't find Straw data directory"
+    return datad
 
 def find_locale_dir():
-    h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
-    if t == 'bin':
-        fp = os.path.join(h, 'share', 'locale')
-        if os.path.isdir(fp):
-            return fp
-    raise "FileNotFoundError", "couldn't find locale data directory"
+    localed = ''
+    if os.environ.has_key("STRAW_LOCALE"):
+        localed = os.environ["STRAW_LOCALE"]
+    else:
+        localed = constants.localedir
+        if not os.path.isdir(localed):
+            raise "FileNotFoundError", "couldn't find locale data directory"
+    return localed
 
 def find_glade_file(libdir=None):
+    gladef = ''
     if os.environ.has_key("STRAW_GLADE"):
-        return os.environ["STRAW_GLADE"]
+        gladef = os.environ["STRAW_GLADE"]
+    else:
+        gladef = os.path.normpath(os.path.join(constants.gladedir, "straw.glade"))
+        if not os.path.isfile(gladef):
+            raise "FileNotFoundError", "couldn't find straw.glade"
+    return gladef
 
-    if libdir is None:
-        libdir = find_lib_dir()
-
-    fp = os.path.join(libdir, 'straw.glade')
-    if os.path.isfile(fp):
-        return fp
-    raise "FileNotFoundError", "couldn't find straw.glade"
+def find_image_dir():
+    if not os.path.isdir(constants.imagedir):
+        raise "FileNotFoundError", "could not find image (pixmaps) directory"
+    return constants.imagedir
 
 def listdiff(l1, l2, test=None):
     if test is not None:
@@ -208,3 +164,33 @@ def listdiff(l1, l2, test=None):
         else:
             inl2.append(e)
     return (common, inl1, inl2)
+
+import string
+import gnomevfs
+import Config
+def url_show(url):
+    config = Config.get_instance()
+    if config.browser_cmd:
+        cmdbin, args = string.splitfields(str(config.browser_cmd), maxsplit=1)
+        link = args % url
+        pid = subprocess.Popen([cmdbin, link]).pid
+        return pid
+    else:
+        return gnomevfs.url_show(url)
+
+
+def set_clipboard_text(text):
+    clipboard = gtk.clipboard_get(selection="CLIPBOARD")
+    clipboard.set_text(text)
+
+def html_replace(exc):
+    """ Python Cookbook 2ed, Section 1.23
+    """
+    if isinstance(exc, (UnicodeDecodeError, UnicodeTranslateError)):
+        s = [ u'&%s;' % codepoint2name[ord(c)] for c in exc.object[exc.start:exc.end]]
+        return ''.join(s), exc.end
+    else:
+        raise TypeError("can't handle %s" % exc.__name__)
+
+import codecs
+codecs.register_error('html_replace', html_replace)
